==> ns-1.0a13.tar.gz: ns-1.0a13/agent.cc <==
/*
 * Copyright (c) 1990-1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: agent.cc,v 1.14 95/11/17 13:19:22 kfall Exp $ (LBL)";
#endif

#include <stdlib.h>

#include "Tcl.h"
#include "agent.h"
#include "node.h"
#include "packet.h"

static class NullAgentMatcher : public Matcher {
public:
	NullAgentMatcher() : Matcher("agent") {}
	virtual TclObject* match(const char* id) {
		if (strcasecmp(id, "null") == 0) {
			return (new Agent(-1));
		}
		return (0);
	}
} matcher_null;

Agent::Agent(int pkttype) : 
	type_(pkttype), node_(0), sport_(-1),
	daddr_(-1), dport_(-1), seqno_(0), size_(0), class_(0)
{
	memset(pending_, 0, sizeof(pending_));
	link_int(0, "class", &class_, 0);
}

Agent::~Agent()
{
}

int Agent::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "addr") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", node_->addr());
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "node") == 0) {
			tcl.result(node_->name());
			return (TCL_OK);
		}
		if (strcmp(argv[1], "dst-addr") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", daddr_);
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "port") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", sport_);
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "dst-port") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", dport_);
			tcl.result(bp);
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "join") == 0) {
			int group = strtol(argv[2], (char**)0, 0);
			node_->join_group(this, group);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "leave") == 0) {
			int group = strtol(argv[2], (char**)0, 0);
			node_->leave_group(this, group);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "node") == 0) {
			node_ = (Node*)TclObject::lookup(argv[2]);
			sport_ = node_->add_agent(this);
			return (TCL_OK);
		}
	} else if (argc == 4) {
		if (strcmp(argv[1], "connect") == 0) {
			daddr_ = strtol(argv[2], (char**)0, 0);
			dport_ = strtol(argv[3], (char**)0, 0);
			if (daddr_ == node_->addr()) {
				tcl.result("ns connect: can't connect to self");
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
	}
	return (NsObject::command(argc, argv));
}

void Agent::sched(double delay, int tno)
{
	if (pending_[tno])
		/*XXX timers botched*/
		abort();

	pending_[tno] = 1;
	Scheduler& s = Scheduler::instance();
	s.schedule(this, &timer_[tno], delay);
}

void Agent::timeout(int)
{
}

void Agent::handle(Event* e)
{
	if (e >= &timer_[0] && e < &timer_[NTIMER]) {
		int tno = e - &timer_[0];
		pending_[tno] = 0;
		timeout(tno);
	} else
		/* otherwise, can only be a packet */
		recv((Packet*)e);
}

void Agent::recv(Packet* p)
{
	/*
	 * didn't expect packet (or we're a null agent?)
	 */
	Packet::free(p);
}

/*
 * allocate a packet and fill in all the generic fields
 */
Packet* Agent::allocpkt(int seqno, int reason) const
{
	Packet* p = Packet::alloc();
	p->type_ = type_;
	p->flags_ = 0;
	p->class_ = class_;
	p->src_ = node_->addr();
	p->sport_ = sport_;
	p->dst_ = daddr_;
	p->dport_ = dport_;
	p->size_ = size_;
	p->seqno_ = seqno;
	p->reason_ = reason;
	return (p);
}

==> ns-1.1.tar.gz: ns-1.1/agent.cc <==
/*
 * Copyright (c) 1990-1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /n/olympus/opt/users/CVS/ns-1.1-master/agent.cc,v 1.2 1997/09/08 20:50:51 wuchang Exp $ (LBL)";
#endif

#include <stdlib.h>

#include "Tcl.h"
#include "agent.h"
#include "node.h"
#include "packet.h"

extern "C" { int strcasecmp(const char*, const char*); }

static class NullAgentMatcher : public Matcher {
public:
	NullAgentMatcher() : Matcher("agent") {}
	virtual TclObject* match(const char* id) {
		if (strcasecmp(id, "null") == 0) {
			return (new Agent(-1));
		}
		return (0);
	}
} matcher_null;

Agent::Agent(int pkttype) : 
	type_(pkttype), node_(0), sport_(-1),
	daddr_(-1), dport_(-1), seqno_(0), size_(0), class_(0)
{
	memset(pending_, 0, sizeof(pending_));
	link_int(0, "class", &class_, 0);
}

Agent::~Agent()
{
}

int Agent::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "addr") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", node_->addr());
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "node") == 0) {
			tcl.result(node_->name());
			return (TCL_OK);
		}
		if (strcmp(argv[1], "dst-addr") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", daddr_);
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "port") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", sport_);
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "dst-port") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", dport_);
			tcl.result(bp);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "seqno") == 0) {
			char* bp = tcl.buffer();
			sprintf(bp, "%d", seqno_);
			tcl.result(bp);
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "join") == 0) {
			int group = strtol(argv[2], (char**)0, 0);
			node_->join_group(this, group);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "leave") == 0) {
			int group = strtol(argv[2], (char**)0, 0);
			node_->leave_group(this, group);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "node") == 0) {
			node_ = (Node*)TclObject::lookup(argv[2]);
			sport_ = node_->add_agent(this);
			return (TCL_OK);
		}
	} else if (argc == 4) {
		if (strcmp(argv[1], "connect") == 0) {
			daddr_ = strtol(argv[2], (char**)0, 0);
			dport_ = strtol(argv[3], (char**)0, 0);
			if (daddr_ == node_->addr()) {
				tcl.result("ns connect: can't connect to self");
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
	}
	return (NsObject::command(argc, argv));
}

void Agent::sched(double delay, int tno)
{
	if (pending_[tno])
		/*XXX timers botched*/
		abort();

	pending_[tno] = 1;
	Scheduler& s = Scheduler::instance();
	s.schedule(this, &timer_[tno], delay);
}

void Agent::timeout(int)
{
}

void Agent::handle(Event* e)
{
	if (e >= &timer_[0] && e < &timer_[NTIMER]) {
		int tno = e - &timer_[0];
		pending_[tno] = 0;
		timeout(tno);
	} else
		/* otherwise, can only be a packet */
		recv((Packet*)e);
}

void Agent::recv(Packet* p)
{
	/*
	 * didn't expect packet (or we're a null agent?)
	 */
	Packet::free(p);
}

/*
 * allocate a packet and fill in all the generic fields
 */
Packet* Agent::allocpkt(int seqno) const
{
	Packet* p = Packet::alloc();
	p->type_ = type_;
	p->flags_ = 0;
	p->class_ = class_;
	p->src_ = node_->addr();
	p->sport_ = sport_;
	p->dst_ = daddr_;
	p->dport_ = dport_;
	p->size_ = size_;
	p->seqno_ = seqno;
	return (p);
}
